@@ -1,12 +1,11 @@
 #__version__ = "4"
-from pkg_resources import get_distribution, DistributionNotFound
+import importlib.metadata
 try:
-    __version__ = get_distribution(__name__).version
-except DistributionNotFound:
+    __version__ = importlib.metadata.version('kilosort')
+except importlib.metadata.PackageNotFoundError:
     # package is not installed
     pass
 
-
 from .utils import PROBE_DIR, DOWNLOADS_DIR
 from .run_kilosort import run_kilosort
 from .parameters import DEFAULT_SETTINGS
