@@ -183,7 +183,8 @@ def kernel2D(x, y, sig = 1):
     Kn = np.exp(-ds / (2*sig**2))
     return Kn
 
-def run(ops, bfile, device=torch.device('cuda'), progress_bar=None):
+def run(ops, bfile, device=torch.device('cuda'), progress_bar=None,
+        clear_cache=False):
     """ this step computes a drift correction model
     it returns vertical correction amplitudes for each batch, and for multiple blocks in a batch if nblocks > 1. 
     """
@@ -194,7 +195,10 @@ def run(ops, bfile, device=torch.device('cuda'), progress_bar=None):
         return ops, None
     
     # the first step is to extract all spikes using the universal templates
-    st, _, ops  = spikedetect.run(ops, bfile, device=device, progress_bar=progress_bar)
+    st, _, ops  = spikedetect.run(
+        ops, bfile, device=device, progress_bar=progress_bar,
+        clear_cache=clear_cache
+        )
 
     # spikes are binned by amplitude and y-position to construct a "fingerprint" for each batch
     F, ysamp = bin_spikes(ops, st)
